#ifndef lapackwrap_h
#define lapackwrap_h

/* =========================== Internals ============================= */

/* DLAMCH - determine double precision machine parameters */
doublereal dlamch_(char *cmach, integer lmach);

/* xLANGE - return the value of the one norm, or the Frobenius norm, or the
 * infinity norm, or the element of largest absolute value of a real matrix A
 * */
doublereal dlange_(char *norm, integer *m, integer *n, doublereal *a,
    integer *lda, doublereal *work, integer lnorm);
doublereal zlange_(char *norm, integer *m, integer *n, doublecomplex *a,
    integer *lda, doublereal *work, integer lnorm);

/* xLASET - initialize an m-by-n matrix A to BETA on the diagonal and
 * ALPHA on the offdiagonals */
int dlaset_(char *uplo, integer *m, integer *n, doublereal *alpha,
    doublereal *beta, doublereal *a, integer *lda, integer luplo);
int zlaset_(char *uplo, integer *m, integer *n, doublecomplex *alpha,
    doublecomplex *beta, doublecomplex *a, integer *lda, integer luplo);

/* =========================== General ============================= */

/* xGEBAK - form the right or left eigenvectors of a real [complex] general
 * matrix by backward transformation on the computed eigenvectors of the
 * balanced matrix output by xGEBAL */
int dgebak_(char *job, char *side, integer *n, integer *ilo, 
    integer *ihi, doublereal *scale, integer *m, doublereal *v,
    integer *ldv, integer *info, integer ljob, integer lside);

int zgebak_(char *job, char *side, integer *n, integer *ilo, 
    integer *ihi, doublereal *scale, integer *m, doublecomplex *v, 
    integer *ldv, integer *info, integer ljob, integer lside);

/* xGEBAL - balance a general real [complex] matrix A */
int dgebal_(char *job, integer *n, doublereal *a, integer *lda,
    integer *ilo, integer *ihi, doublereal *scale, integer *info,
    integer ljob);

int zgebal_(char *job, integer *n, doublecomplex *a, integer *lda,
    integer *ilo, integer *ihi, doublereal *scale, integer *info,
    integer ljob);

/* xGECON - estimate the reciprocal of the condition number of a general real
 * [complex] matrix A, in either the 1-norm or the infinity-norm, using the LU
 * factorization computed by xGETRF */
int dgecon_(char *norm, integer *n, doublereal *a, integer *lda,
    doublereal *anorm, doublereal *rcond, doublereal *work,
    integer *iwork, integer *info, integer lnorm);

int zgecon_(char *norm, integer *n, doublecomplex *a, integer *lda,
    doublereal *anorm, doublereal *rcond, doublecomplex *work,
    doublereal *rwork, integer *info, integer lnorm); 

/* xGEEV - compute for an N-by-N real [complex] nonsymmetric matrix A, the
 * eigenvalues and, optionally, the left and/or right eigenvectors */
int dgeev_(char *jobvl, char *jobvr, integer *n, doublereal *a,
    integer *lda, doublereal *wr, doublereal *wi, doublereal *vl, 
    integer *ldvl, doublereal *vr, integer *ldvr, doublereal *work, 
    integer *lwork, integer *info, integer ljobvl, integer ljobvr);

int zgeev_(char *jobvl, char *jobvr, integer *n, doublecomplex *a,
    integer *lda, doublecomplex *w, doublecomplex *vl, integer *ldvl,
    doublecomplex *vr, integer *ldvr, doublecomplex *work, 
    integer *lwork, doublereal *rwork, integer *info, integer ljobvl,
    integer ljobvr);
 
/* xGEHRD - reduce a real [complex] general matrix A to upper Hessenberg form
 * H by an orthogonal similarity transformation */
int dgehrd_(integer *n, integer *ilo, integer *ihi, doublereal *a,
    integer *lda, doublereal *tau, doublereal *work, 
    integer *lwork, integer *info);

int zgehrd_(integer *n, integer *ilo, integer *ihi, doublecomplex *a,
    integer *lda, doublecomplex *tau, doublecomplex *work,
    integer *lwork, integer *info);
 
/* xGELQF - compute an LQ factorization of a real [complex] M-by-N matrix A */
int dgelqf_(integer *m, integer *n, doublereal *a, integer *lda,
    doublereal *tau, doublereal *work, integer *lwork, integer *info);

int zgelqf_(integer *m, integer *n, doublecomplex *a, integer *lda,
    doublecomplex *tau, doublecomplex *work, integer *lwork,
    integer *info);
 
/* xGELS - solve overdetermined or underdetermined real [complex] linear
 * systems involving an M-by-N matrix A, or its transpose, using a QR or LQ
 * factorization of A */
int dgels_(char *trans, integer *m, integer *n, integer *nrhs,
    doublereal *a, integer *lda, doublereal *b, integer *ldb, 
    doublereal *work, integer *lwork, integer *info, integer ltrans);

int zgels_(char *trans, integer *m, integer *n, integer *nrhs,
    doublecomplex *a, integer *lda, doublecomplex *b, integer *ldb, 
    doublecomplex *work, integer *lwork, integer *info, integer ltrans);

/* xGELSD - compute the minimum-norm solution to a real [complex] linear
 * least squares problem */
int dgelsd_(integer *m, integer *n, integer *nrhs, doublereal *a,
    integer *lda, doublereal *b, integer *ldb, doublereal *s,
    doublereal *rcond, integer *rank, doublereal *work, integer *lwork,
    integer *iwork, integer *info);

int zgelsd_(integer *m, integer *n, integer *nrhs, doublecomplex *a,
    integer *lda, doublecomplex *b, integer *ldb, doublereal *s,
    doublereal *rcond, integer *rank, doublecomplex *work, integer *lwork,
    doublereal *rwork, integer *iwork, integer *info);

/* xGELSY  - compute the minimum-norm solution to a real [complex] linear
 * least squares problem using complete orthogonal decomposition */
int dgelsy_(integer *m, integer *n, integer *nrhs, doublereal *a,
    integer *lda, doublereal *b, integer *ldb, integer *jpvt,
    doublereal *rcond, integer *rank, doublereal *work, integer *lwork,
    integer *info);

int zgelsy_(integer *m, integer *n, integer *nrhs, doublecomplex *a,
    integer *lda, doublecomplex *b, integer *ldb, integer *jpvt,
    doublereal *rcond, integer *rank, doublecomplex *work, integer *lwork,
    doublereal *rwork, integer *info);

/* xGELSS  - compute the minimum norm solution to a real [complex] linear
 * least squares problem using SVD decomposition */
int dgelss_(integer *m, integer *n, integer *nrhs, doublereal *a,
    integer *lda, doublereal *b, integer *ldb, doublereal *s,
    doublereal *rcond, integer *rank, doublereal *work, integer *lwork,
    integer *info);

int zgelss_(integer *m, integer *n, integer *nrhs, doublecomplex *a,
    integer *lda, doublecomplex *b, integer *ldb, doublereal *s,
    doublereal *rcond, integer *rank, doublecomplex *work, integer *lwork,
    doublereal *rwork, integer *info);

/* xGEQP3 - compute a QR factorization with column pivoting of a matrix A */
int dgeqp3_(integer *m, integer *n, doublereal *a, integer *lda,
    integer *jpvt, doublereal *tau, doublereal *work, integer *lwork,
    integer *info);

int zgeqp3_(integer *m, integer *n, doublecomplex *a, integer *lda,
    integer *jpvt, doublecomplex *tau, doublecomplex *work, 
    integer *lwork, doublereal *rwork, integer *info);

/* xGEQRF - compute a QR factorization of a real [complex] M-by-N matrix A */
int dgeqrf_(integer *m, integer *n, doublereal *a, integer *lda,
    doublereal *tau, doublereal *work, integer *lwork, integer *info);

int zgeqrf_(integer *m, integer *n, doublecomplex *a, integer *lda,
    doublecomplex *tau, doublecomplex *work, integer *lwork,
    integer *info);
 
/* xGESV - compute the solution to a real [complex] system of linear equations
 * A * X = B, where A is an N-by-N matrix and X and B are N-by-NRHS matrices */
int dgesv_(integer *n, integer *nrhs, doublereal *a, integer *lda,
    integer *ipiv, doublereal *b, integer *ldb, integer *info);

int zgesv_(integer *n, integer *nrhs, doublecomplex *a, integer *lda,
    integer *ipiv, doublecomplex *b, integer *ldb, integer *info);
 
/* xGESVD - compute the singular value decomposition (SVD) of a real [complex]
 * M-by-N matrix A, optionally computing the left and/or right singular
 * vectors */
int dgesvd_(char *jobu, char *jobvt, integer *m, integer *n, 
    doublereal *a, integer *lda, doublereal *s, doublereal *u,
    integer *ldu, doublereal *vt, integer *ldvt, doublereal *work,
    integer *lwork, integer *info, integer ljobu, integer ljobvt);

int zgesvd_(char *jobu, char *jobvt, integer *m, integer *n, 
    doublecomplex *a, integer *lda, doublereal *s, doublecomplex *u,
    integer *ldu, doublecomplex *vt, integer *ldvt, doublecomplex *work,
    integer *lwork, doublereal *rwork, integer *info, integer ljobu,
    integer ljobvt);

/* xGETRF - compute an LU factorization of a general M-by-N matrix A using
 * partial pivoting with row interchanges */
int dgetrf_(integer *m, integer *n, doublereal *a, integer *lda,
    integer *ipiv, integer *info);

int zgetrf_(integer *m, integer *n, doublecomplex *a, integer *lda,
    integer *ipiv, integer *info);

/* xGETRI - compute the inverse of a matrix using the LU factorization
 * computed by xGETRF */
int dgetri_(integer *n, doublereal *a, integer *lda, integer *ipiv,
    doublereal *work, integer *lwork, integer *info);

int zgetri_(integer *n, doublecomplex *a, integer *lda, integer *ipiv,
    doublecomplex *work, integer *lwork, integer *info);

/* xGETRS - solve a system of linear equations A * X = B or A' * X = B with a
 * general N-by-N matrix A using the LU factorization computed by xGETRF */
int dgetrs_(char *trans, integer *n, integer *nrhs, doublereal *a,
    integer *lda, integer *ipiv, doublereal *b, integer *ldb,
    integer *info, integer ltrans);

int zgetrs_(char *trans, integer *n, integer *nrhs, doublecomplex *a,
    integer *lda, integer *ipiv, doublecomplex *b, integer *ldb,
    integer *info, integer ltrans);
 
/* DORGHR - generate a real orthogonal matrix Q which is defined as the
 * product of IHI-ILO elementary reflectors of order N, as returned by DGEHRD
 * */
int dorghr_(integer *n, integer *ilo, integer *ihi, doublereal *a,
    integer *lda, doublereal *tau, doublereal *work, integer *lwork,
    integer *info);

/* DORGLQ - generate an M-by-N real matrix Q with orthonormal rows, which is
 * defined as the first M rows of a product of K elementary reflectors of
 * order N as returned by DGELQF */
int dorglq_(integer *m, integer *n, integer *k, doublereal *a,
    integer *lda, doublereal *tau, doublereal *work, integer *lwork, 
    integer *info);

/* xOR[UN]GQR - generate an M-by-N real [complex] matrix Q with orthonormal
 * columns, which is defined as the first N columns of a product of K
 * elementary reflectors of order M as returned by xGEQRF */
int dorgqr_(integer *m, integer *n, integer *k, doublereal *a,
    integer *lda, doublereal *tau, doublereal *work, integer *lwork, 
    integer *info);

int zungqr_(integer *m, integer *n, integer *k, doublecomplex *a,
    integer *lda, doublecomplex *tau, doublecomplex *work, integer *lwork, 
    integer *info);

/* DORMHR - overwrite the general real M-by-N matrix C with SIDE = 'L' SIDE =
 * 'R' TRANS = 'N': Q * C C * Q TRANS = 'T': Q**T * C    C * Q**T where Q is a
 * real orthogonal matrix of order nq, with nq = m if SIDE = 'L' and nq = n if
 * SIDE = 'R'. Q is defined as the product of IHI-ILO elementary reflectors,
 * as returned by DGEHRD */
int dormhr_(char *side, char *trans, integer *m, integer *n, 
  integer *ilo, integer *ihi, doublereal *a, integer *lda, doublereal *
  tau, doublereal *c, integer *ldc, doublereal *work, integer *lwork, 
  integer *info, integer lside, integer ltrans);

/* DORMLQ - overwrite the general real M-by-N matrix C with SIDE = 'L' SIDE =
 * 'R' TRANS = 'N': Q * C C * Q TRANS = 'T': Q**T * C   C * Q**T where Q is a
 * real orthogonal matrix defined as the product of k elementary reflectors,
 * as returned by DGELQF */
int dormlq_(char *side, char *trans, integer *m, integer *n, 
    integer *k, doublereal *a, integer *lda, doublereal *tau,
    doublereal *c, integer *ldc, doublereal *work, integer *lwork,
    integer *info, integer lside, integer ltrans);

/* xOR[UN]MQR - overwrite the general real M-by-N matrix C with SIDE = 'L' SIDE =
 * 'R' TRANS = 'N': Q * C C * Q TRANS = 'T': Q**T * C   C * Q**T where Q is a
 * real orthogonal matrix defined as the product of k elementary reflectors,
 * as returned by xGEQRF */
int dormqr_(char *side, char *trans, integer *m, integer *n, 
    integer *k, doublereal *a, integer *lda, doublereal *tau,
    doublereal *c, integer *ldc, doublereal *work, integer *lwork,
    integer *info, integer lside, integer ltrans);

int zunmqr_(char *side, char *trans, integer *m, integer *n, 
    integer *k, doublecomplex *a, integer *lda, doublecomplex *tau,
    doublecomplex *c, integer *ldc, doublecomplex *work, integer *lwork,
    integer *info, integer lside, integer ltrans);


/* =========================== Posdef Symm ============================= */

/* xPOCON - estimate the reciprocal of the condition number (in the 1-norm) of
 * a real [complex] symmetric [Hermitian] positive definite matrix using the
 * Cholesky factorization A = U**H*U or A = L*L**H computed by xPOTRF */
int dpocon_(char *uplo, integer *n, doublereal *a, integer *lda,
    doublereal *anorm, doublereal *rcond, doublereal *work,
    integer *iwork, integer *info, integer luplo);

int zpocon_(char *uplo, integer *n, doublecomplex *a, integer *lda,
    doublereal *anorm, doublereal *rcond, doublecomplex *work,
    doublereal *rwork, integer *info, integer luplo);
 
/* xPOTRF - compute the Cholesky factorization of a real [complex] symmetric
 * [Hermitian] positive definite matrix A */
int dpotrf_(char *uplo, integer *n, doublereal *a, integer *lda,
    integer *info, integer luplo);
 
int zpotrf_(char *uplo, integer *n, doublecomplex *a, integer *lda,
    integer *info, integer luplo);

/* xPOTRI - compute  the  inverse of a real [complex] symmetric [Hermitian]
 * positive definite matrix A using the Cholesky factorization A = U**H*U or
 * A  = L*L**H computed by xPOTRF */
int dpotri_(char *uplo, integer *n, doublereal *a, integer *lda,
    integer *info, integer luplo);

int zpotri_(char *uplo, integer *n, doublecomplex *a, integer *lda,
    integer *info, integer luplo);
 
/* xPOTRS - solve a system of linear equations A*X = B with a symmetric
 * [Hermitian] positive definite matrix A using the Cholesky factorization
 * A = U**H*U or A = L*L**H computed by xPOTRF */
int dpotrs_(char *uplo, integer *n, integer *nrhs, doublereal *a,
    integer *lda, doublereal *b, integer *ldb, integer *info, integer luplo);

int zpotrs_(char *uplo, integer *n, integer *nrhs, doublecomplex *a,
    integer *lda, doublecomplex *b, integer *ldb, integer *info, integer luplo);


/* =========================== Symmetric ============================= */

/* xSY[HE]CON - estimate the reciprocal of the condition number (in the 1-norm)
 * of a real symmetric [SY] or complex Hermitian [HE] matrix A using the
 * factorization A = U*D*U**H or A = L*D*L**H computed by xSY[HE]TRF */
int dsycon_(char *uplo, integer *n, doublereal *a, integer *lda,
    integer *ipiv, doublereal *anorm, doublereal *rcond, doublereal *work,
    integer *iwork, integer *info, integer luplo);

int zhecon_(char *uplo, integer *n, doublecomplex *a, integer *lda,
    integer *ipiv, doublereal *anorm, doublereal *rcond, 
    doublecomplex *work, integer *info, integer luplo);
 
/* xSY[HE]EV - compute all eigenvalues and, optionally, eigenvectors of a real
 * symmetric [SY] or complex Hermitian [HE] matrix A */
int dsyev_(char *jobz, char *uplo, integer *n, doublereal *a,
    integer *lda, doublereal *w, doublereal *work, integer *lwork,
    integer *info, integer ljobz, integer luplo);

int zheev_(char *jobz, char *uplo, integer *n, doublecomplex *a,
    integer *lda, doublereal *w, doublecomplex *work, integer *lwork, 
    doublereal *rwork, integer *info, integer ljobz, integer luplo);

/* xSY[HE]SV - compute the solution to a real [complex] system of linear
 * equations A * X = B, where A is an N-by-N symmetric [Hermitian] matrix and
 * X and B are N-by-NRHS matrices */
int dsysv_(char *uplo, integer *n, integer *nrhs, doublereal *a,
    integer *lda, integer *ipiv, doublereal *b, integer *ldb, 
    doublereal *work, integer *lwork, integer *info, integer luplo);

int zhesv_(char *uplo, integer *n, integer *nrhs, doublecomplex *a,
    integer *lda, integer *ipiv, doublecomplex *b, integer *ldb,
    doublecomplex *work, integer *lwork, integer *info, integer luplo);
 
/* xSY[HE]TRF - compute the factorization of a real symmetric [SY] or complex
 * Hermitian [HE] matrix A using the Bunch-Kaufman diagonal pivoting method */
int dsytrf_(char *uplo, integer *n, doublereal *a, integer *lda,
    integer *ipiv, doublereal *work, integer *lwork, integer *info,
    integer luplo);

int zhetrf_(char *uplo, integer *n, doublecomplex *a, integer *lda,
    integer *ipiv, doublecomplex *work, integer *lwork, integer *info,
    integer luplo);
 
/* xSY[HE]TRI - compute the inverse of a real symmetric [SY] or complex
 * Hermitian [HE] indefinite matrix A using the factorization A = U*D*U**H
 * or A = L*D*L**H computed by xSY[HE]TRF */
int dsytri_(char *uplo, integer *n, doublereal *a, integer *lda,
    integer *ipiv, doublereal *work, integer *info, integer luplo);

int zhetri_(char *uplo, integer *n, doublecomplex *a, integer *lda,
    integer *ipiv, doublecomplex *work, integer *info, integer luplo);
 
/* xSY[HE]TRS - solve a system of linear equations A*X = B with a real
 * symmetric [SY] or complex Hermitian [HE] matrix A using the factorization
 * A = U*D*U**H or A = L*D*L**H computed by xSY[HE]TRF */
int dsytrs_(char *uplo, integer *n, integer *nrhs, doublereal *a,
    integer *lda, integer *ipiv, doublereal *b, integer *ldb,
    integer *info, integer luplo);

int zhetrs_(char *uplo, integer *n, integer *nrhs, doublecomplex *a,
    integer *lda, integer *ipiv, doublecomplex *b, integer *ldb,
    integer *info, integer luplo);
 

/* =========================== Triangular ============================= */

/* xTRCON - estimate the reciprocal of the condition number of a triangular
 * matrix A, in either the 1-norm or the infinity-norm */
int dtrcon_(char *norm, char *uplo, char *diag, integer *n, 
    doublereal *a, integer *lda, doublereal *rcond, doublereal *work, 
    integer *iwork, integer *info, integer lnorm, integer luplo,
    integer ldiag);

int ztrcon_(char *norm, char *uplo, char *diag, integer *n, 
    doublecomplex *a, integer *lda, doublereal *rcond,
    doublecomplex *work, doublereal *rwork, integer *info, integer lnorm,
    integer luplo, integer ldiag);
 
/* xTRTRI - compute the inverse of a real [complex] upper or lower triangular
 * matrix A */
int dtrtri_(char *uplo, char *diag, integer *n, doublereal *a,
    integer *lda, integer *info, integer luplo, integer ldiag);

int ztrtri_(char *uplo, char *diag, integer *n, doublecomplex *a,
    integer *lda, integer *info, integer luplo, integer ldiag);

/* xTRTRS - solve a triangular system of the form A * X = B or A**H * X = B,
 * where A is a triangular matrix of order N, and B is an N-by-NRHS matrix. A
 * check is made to verify that A is nonsingular */
int dtrtrs_(char *uplo, char *trans, char *diag, integer *n,
    integer *nrhs, doublereal *a, integer *lda, doublereal *b,
    integer *ldb, integer *info, integer luplo, integer ltrans,
    integer ldiag);

int ztrtrs_(char *uplo, char *trans, char *diag, integer *n,
    integer *nrhs, doublecomplex *a, integer *lda, doublecomplex *b, 
    integer *ldb, integer *info, integer luplo, integer ltrans,
    integer ldiag);
 
#endif
