#ifndef ranlib_h
#define ranlib_h

/* Constants */
#define LUARNG_MAXSTATES (624)  /* _N_ in mt19937ar.c */
#define LUARNG_SEED (5489UL)  /* default initial seed in mt199937ar.c */

/* Struct to hold state vector and current position */
typedef struct {
  unsigned long v[LUARNG_MAXSTATES];  /* the array for the state vector */
  int i;  /* i==N+1 means v[N] is not initialized */
} lua_RNG;

/* Uniform deviates are generated by Mersenne Twister method,
 * algorithm mt19937 by Makoto Matsumoto and Takuji Nishimura.
 * Check mt.c for details. */
void init_genrand(lua_RNG *o, unsigned long s);
void init_by_array(lua_RNG *o, unsigned long init_key[], int key_length);
unsigned long genrand_int32(lua_RNG *o);
long genrand_int31(lua_RNG *o);
double genrand_real1(lua_RNG *o);
double genrand_real2(lua_RNG *o);
double genrand_real3(lua_RNG *o);
double genrand_res53(lua_RNG *o); 

#define ranf(o) genrand_real3(o)
#define ignlgi(o) genrand_int31(o)
/* All other deviates are computed from an adapted ranlib.c from Netlib */
double genbet(lua_RNG *o,double aa,double bb);
double genchi(lua_RNG *o,double df);
double genexp(lua_RNG *o,double av);
double genf(lua_RNG *o,double dfn,double dfd);
double gengam(lua_RNG *o,double a,double r);
void genmul(lua_RNG *o,long n,double *p,long ncat,long *ix);
double gennch(lua_RNG *o,double df,double xnonc);
double gennf(lua_RNG *o,double dfn,double dfd,double xnonc);
double gennor(lua_RNG *o,double av,double sd);
void genprm(lua_RNG *o,long *iarray,int larray);
double genunf(lua_RNG *o,double low,double high);
long ignbin(lua_RNG *o,long n,double pp);
long ignnbn(lua_RNG *o,long n,double p);
long ignpoi(lua_RNG *o,double mu);
long ignuin(lua_RNG *o,long low,long high);

#endif
